import { Observable, Subject, merge } from 'rxjs';
import { switchMap, shareReplay, debounceTime } from 'rxjs/operators';

export class ObserveService<T> {
    private updateStream: Subject<void>;
    public value$: Observable<T>;

    constructor(request: ()=>Observable<T>) {
        this.updateStream = new Subject<void>();
        const initialCurrentResponse$ = request();
        const reload$ = this.updateStream.pipe(debounceTime(1000), switchMap(() => request()));
        const final$ = merge(initialCurrentResponse$, reload$).pipe(shareReplay(1));
        this.value$ = final$;
    }

    public next() {
        this.updateStream.next();
    }
}
